﻿
using System.Collections.Generic;
using Microsoft.Xna.Framework;

namespace XELF {

	static partial class GeometryHelper {
		/// <summary>
		/// 点群の凸包を求める。
		/// </summary>
		/// <param name="vertices">点群</param>
		/// <returns>凸包の点リスト</returns>
		public static List<Vector2> ConvexHull(this List<Vector2> vertices) {
			List<Vector2> hull = new List<Vector2>();
			if (vertices.Count <= 0)
				return hull;

			// 最小のYを持つ点のインデックスを得る。
			int first = 0;
			{
				float y = float.PositiveInfinity;
				for (int i = 0; i < vertices.Count; i++) {
					if (vertices[i].Y < y) {
						first = i;
						y = vertices[i].Y;
					}
				}
			}

			var oldPoint = vertices[first];
			var oldDirection = Vector2.UnitX;

			hull.Clear();
			hull.Add(oldPoint);

			for (int i = 1; i < vertices.Count; i++) {
				var maximumCos = float.MinValue;
				var index = 0;
				var direction = Vector2.Zero;
				for (int j = 0; j < vertices.Count; j++) {
					if (vertices[j] == oldPoint)
						continue;
					Vector2 d = vertices[j] - oldPoint;
					d.Normalize();
					float cos;
					Vector2.Dot(ref oldDirection, ref d, out cos);
					if (cos > maximumCos) { // 方向ベクトルの変化が最小（最小の角度θ、つまり最大のcosθ）になる点を探す。
						index = j;
						maximumCos = cos;
						direction = d;
					}
				}
				if (index == first)
					return hull;
				hull.Add(vertices[index]);
				oldPoint = vertices[index];
				oldDirection = direction;
			}
			return hull;
		}
	}

}
