using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using XELF;

namespace PCA {

	public class PCAGame2 : Microsoft.Xna.Framework.Game {
		GraphicsDeviceManager graphics;
		SpriteBatch spriteBatch;
		VectorGraphics vectorGraphics;

		Camera camera = new Camera();
		List<Vector2> Points { get; set; }
		PrincipalComponentAnalysis2 pca = new PrincipalComponentAnalysis2();

		MouseState previous;
		MouseState current;
		Matrix obb;
		List<Vector2> hull;

		public PCAGame2() {
			graphics = new GraphicsDeviceManager(this);
			Content.RootDirectory = "Content";
			graphics.PreferMultiSampling = true;
			IsMouseVisible = true;
			Points = new List<Vector2>();
			Components.Add(vectorGraphics = new VectorGraphics(this));
		}

		protected override void LoadContent() {
			spriteBatch = new SpriteBatch(GraphicsDevice);
			vectorGraphics.Camera = camera;
			camera.View = Matrix.CreateLookAt(Vector3.UnitZ, Vector3.Zero, Vector3.UnitY);
			camera.Projection = Matrix.CreateOrthographicOffCenter(0, 800, 600, 0, 0, 10);
		}

		protected override void Update(GameTime gameTime) {

			if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
				this.Exit();

			previous = current;
			current = Mouse.GetState();

			if (current.LeftButton == ButtonState.Pressed) {
				Points.Add(new Vector2(current.X, current.Y));
			}
			if (current.RightButton == ButtonState.Pressed && previous.RightButton == ButtonState.Released) {
				Points.Clear();
			}

			hull = Points.ConvexHull();
			pca.Compute(hull);

			obb = hull.OBB();

			base.Update(gameTime);
		}

		protected override void Draw(GameTime gameTime) {
			GraphicsDevice.Clear(Color.CornflowerBlue);

			vectorGraphics.Begin();

			foreach (var point in Points) {
				vectorGraphics.DrawPoint(point, Color.White);
			}

			var radius = Points.Radius(pca.Centroid);
			Vector2 d1 = new Vector2(pca.Principal1.Y, -pca.Principal1.X);
			Vector2 d2 = new Vector2(pca.Principal2.Y, -pca.Principal2.X);
			var e1 = (float)pca.Eigenvectors[0];
			var e2 = (float)pca.Eigenvectors[1];
			d1 *= radius;
			d2 *= (float)Math.Sqrt(e1 / e2) * radius;
			vectorGraphics.DrawLine(pca.Centroid - d1, Color.Red, pca.Centroid + d1, Color.Red);
			vectorGraphics.DrawLine(pca.Centroid - d2, Color.Green, pca.Centroid + d2, Color.Green);

			Matrix transform = new Matrix(
				d1.X, d1.Y, 0, 0,
				d2.X, d2.Y, 0, 0,
				0, 0, 1, 0,
				pca.Centroid.X, pca.Centroid.Y, 0, 1);
			vectorGraphics.DrawEllipse(pca.Centroid, hull.Radius(pca.Centroid), Color.Yellow);
			vectorGraphics.DrawRectangle(obb, Color.Black);

			vectorGraphics.DrawClosedLine(hull.ToArray(), Color.Orange);

			vectorGraphics.End();

			base.Draw(gameTime);
		}
	}
}
